' USB sample program for PIC18F4550 CDC serial port emulation
'  Requires PBP 2.60 or later

'  Compilation of this program requires that specific support files be
'  available in the source directory.  For detailed information, see
'  the file PBP\USB18\USB.TXT.


	
	Include	"cdc_desc.bas"	' Include the HID descriptors

#CONFIG
	CONFIG  PLLDIV = 1			; 4MHz input 
;	CONFIG  OSC = HSPLL           ; HS+PLL, USB-HS+PLL 18F26J50
    CONFIG  CPUDIV = OSC1_PLL2    ; [Primary Oscillator Src: /1][96 MHz PLL Src: /2] 182455
    CONFIG  USBDIV = 2            ; USB clock source comes from the 96 MHz PLL divided by 2 182455
	CONFIG  FOSC = HSPLL_HS       ; HS oscillator, PLL enabled (HSPLL) 182455
	CONFIG  LVP = OFF			; 182455
	CONFIG WDT = OFF	    ;WDT disabled (control is placed on the SWDTEN bit) 182455
	CONFIG VREGEN = ON	    ;USB voltage regulator enabled 182455
    CONFIG  DEBUG = ON           ; Enabled	
#ENDCONFIG

Define  OSC     48

EP3Size Con 64
MinCharge Con 12000
FullCharge Con 17000
pixel Con 13			; 1 pixel=6 mm
RampIncrement	Con	100

bufferIn	Var	Byte[EP3Size]
RBbufferIn	Var	Byte[EP3Size]
RBbufferOut	Var	Byte[5]
RROSbufferIn	Var	Byte[5]
RROSbufferOut	Var	Byte[2]
RBbufferInHead	Var	Byte
RBbufferInTail	Var	Byte
cnt	Var	Byte
Ccnt Var Byte
BumperStatus Var Byte
InfraredStatus Var Byte
LineSensorStatus Var Byte
RBCommand Var Byte
RBData Var Byte
x Var Byte
MSByte Var Byte
LSByte Var Byte
RequestedDistance Var Word	;SeekDistance
CreateDistance Var Word	;ActualDistance
CreateVelocity Var Word ;Drive
RequestedAngle Var Word	;SeekAngle
CreateAngle Var Word	;ActualAngle
CreateRadius Var Word	;Drive
CreateSpeed Var Word    ; Speed
PresentVolts Var Word
PresentCharge Var Word
MaxCharge Var Word
CreateIR Var Byte
CreateBump Var Byte
CreateLCliff Var Byte
CreateFLCliff Var Byte
CreateRCliff Var Byte
CreateFRCliff Var Byte
CreateVWall Var Byte
CreateOverC Var Byte
CreateDummy Var Byte
CreateColor Var Byte
CreateWall Var Byte
DebugMode Var Byte
TranslateMode Var Byte
WheelRatio Var Byte
SongNumber Var Byte
NumberOfNotes Var Byte
NoteNumber Var Byte[16]
NoteDuration Var Byte[16]
Parameter Var Byte
Dummy Var Byte
Percent Var Byte
BaudSwitch Var Word

LED1 Var PORTA.0						; 1=Command Request, 0=Command Response
LED2 Var PORTA.1						; USB Connected State
LED3 Var PORTA.2						; 1=Translate, 0=Pass through
LED4 Var PORTA.3						; 1=Debug All 0=Debug Timeout Only
Out Var PORTA.4

SW1A Var PORTB.0						; Baud selection: 0=1200, 1=2400, 2=4800
SW1B Var PORTB.1						; 3=9600, 4=19200, 5=37400
SW1C Var PORTB.2						; 6=57600, 7=115200
SW1D Var PORTB.3						; not used
SW2 Var PORTB.4							; Debug ON/OFF
SW3 Var PORTB.5							; Translate ON/OFF

TRISA = %11100000	' LEDs outputs
TRISC.6	= 0	' TX as outputs
ADCON1 = %00001111	' All digital
INTCON2 = INTCON2 & %01111111	; enable PORTB pullups

UCFG = UCFG | %00010000	' Enable USB pullups

	OSCTUNE.6 = 1		' Enable PLL for 18F87J50 family
	Pause 10
	TranslateMode = 0
	DebugMode = 0
	LOW LED1							; 1=Command Request, 0=Command Response
	LOW LED2							; USB Connected State
	LOW LED3         					; 1=Translate, 0=Pass through
	LOW LED4         					; 1=Debug All 0=Debug Timeout Only
	Ccnt = 0
	RBbufferInHead = 0
	RBbufferInTail = 0
	CreateAngle = 0	
	CreateDistance = 0		
	CreateRadius = $FFFF		
	CreateVelocity = 0		
	
	BaudSwitch = PORTB & 7 ^ 7
	lookup2 BaudSwitch, [12, 24, 48, 96, 192, 374, 576, 1152],BaudSwitch
'SPBRG=(FOSC/(64 * Baud)-1
	poke SPBRG,(7500/BaudSwitch)-1
	
' Set receive register to receiver enabled	
DEFINE HSER_RCSTA 90h
' Set transmit register to transmitter enabled
DEFINE HSER_TXSTA 20h ; use 20h for BRGH=0 or 24h for BRGH=1
' Set baud rate
'DEFINE HSER_BAUD 37400
' Set DEBUG pin port
DEFINE DEBUG_REG PORTA
' Set DEBUG pin bit
DEFINE DEBUG_BIT 4
' Set DEBUG baud rate
DEFINE DEBUG_BAUD 19200
' Set DEBUG mode: 0=true, 1=inverted
DEFINE DEBUG_MODE 1

	debug "BlueTooth Baud=",#BaudSwitch,"00",13,10
	debug "Debug Mode=OFF",13,10		; Debug Timeout Only
	debug "Translate Mode=OFF",13,10	; No Translation	
	USBInit			' Get USB going
	
USBWait:	
	pauseus 200
	USBService		' Must service USB regularly
	peek($427),x
	if x != $20 then goto USBWait
	x=0
	
USBWait1:
	pause 10
	USBService		' Must service USB regularly
	x=x+1
	if x < 50 then goto USBWait1
			
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
CkkSW1:
	Peek RCSTA, x
	if x.1 = 0 then CkkSW1Continue
	Poke RCSTA, x & %11101111
	Poke RCSTA, x | %00010000
	 
CkkSW1Continue:
	peek($427),x						; USBDeviceState
	if x = $20 then
		high LED2
	else
		low LED2
	endif
	
	if SW2=0 then 
		DebugMode = DebugMode ^ 1
		while SW2 = 0
		wend
		if DebugMode = 0 then
			HIGH LED4
			debug "Debug Mode=ON",13,10		; Debug
		else
			LOW LED4						
			debug "Debug Mode=OFF",13,10	; Debug Timeout Only		 
		endif		
	endif
		
	if SW3=0 then 
		TranslateMode = TranslateMode ^ 1
		while SW3 = 0
		wend
		if TranslateMode = 0 then
			HIGH LED3						
			debug "Translate Mode=ON",13,10	; Translated 	
			goto Main
		else
			LOW LED3						
			debug "Translate Mode=OFF",13,10; No Translation
		endif		
	endif
	
	if TranslateMode = 0 then
		goto Main						
	else
		goto BT
	endif
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
BT:	
 	USBService		' Must service USB regularly
	cnt = EP3Size		' Specify input buffer size
	USBIn 3, bufferIn, cnt, BTResponse
	if cnt=64 then goto CkkSW1	
	HSEROUT [STR bufferIn\cnt]

BTResponse:
	Peek PIR1, x
	if x.5 = 0 then goto CkkSW1
	HSERIN [RBbufferOut]		
	USBService		' Must service USB regularly	
	USBOut 3, RBbufferOut, 1, BTResponse
	goto BTResponse
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Main:	
	gosub ServiceUSB
	
Chk4Command:	
	if RBbufferInHead = RBbufferInTail then MainExit
	if RBbufferInHead > RBbufferInTail then
		if RBbufferInHead-RBbufferInTail > 1 then GetCommand
	else
		if RBbufferInHead+EP3Size-RBbufferInTail > 1 then GetCommand

	endif

MainExit:	
	goto CkkSW1	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
GetCommand:	
	RBCommand = RBbufferIn[RBbufferInTail]
	RBbufferInTail = RBbufferInTail+1
	if RBbufferInTail = EP3Size then RBbufferInTail=0
	
	RBData = RBbufferIn[RBbufferInTail]
	RBbufferInTail = RBbufferInTail+1
	if RBbufferInTail = EP3Size then RBbufferInTail=0		

;	if RBCommand="B" and RBData="T" then 
;		Mode=1
;		goto CkkSW1	
;	endif
				
OutputToRROS:
	gosub ServiceUSB
	if DebugMode = 0 then debug "RB-> ", HEX RBCommand, " ", HEX RBData, 13, 10
	on RBCommand/16 gosub Code0_F, Code10_1F, Code20_2F, Code30_3F, Code40_4F, Code50_5F, Code60_6F, Code70_7F,Code80_8F, Code90_9F, CodeA0_AF, CodeB0_BF, CodeC0_CF, CodeD0_DF, CodeE0_EF, CodeF0_FF 	

	goto Main


GetReply:
	gosub ServiceUSB	
	HSERIN 100, FakeReply, [STR RROSbufferIn\5]

OutputToRB:
	gosub ServiceUSB	
	for x = 0 to 4
		RBbufferOut[x] = RROSbufferIn[x] 
	next x
	goto outloop	
		
FakeReply:
	ARRAYWRITE RBbufferOut, [0,0,0,0,0]
	
outloop:	
	USBService		' Must service USB regularly
	USBOut 3, RBbufferOut, 5, outloop
	if DebugMode = 0 then debug "->RB ", HEX RBbufferOut[0], " ", HEX RBbufferOut[1], " ", HEX RBbufferOut[2], " ", HEX RBbufferOut[3], " ", HEX RBbufferOut[4], 13, 10
	LOW LED1							; Command Finished
	return		

ServiceUSB:
 	USBService		' Must service USB regularly
	cnt = EP3Size		' Specify input buffer size
	USBIn 3, bufferIn, cnt, ServiceUSBExit

	
	for x = 0 to cnt-1
		RBbufferIn[RBbufferInHead] = bufferIn[x]
		RBbufferInHead = RBbufferInHead+1
		if RBbufferInHead = EP3Size then RBbufferInHead=0
	next x
	HIGH LED1							; Command Request	
;
ServiceUSBExit:
	return
	
Code0_F:
	on RBCommand goto Code0, Code1, Code2, Code3, Code4, Code5, Code6, Code7, Code8, Code9, CodeA, CodeB, CodeC, CodeD, CodeE, CodeF
	
Code0:
	goto FakeReply
	
Code1:
	goto FakeReply
	
Code2:
	goto FakeReply
	
Code3:	; 3 Locate, RBData=0-255
	gosub Start
	goto OutputToRB
	
Code4:
	goto FakeReply
	
Code5:
	goto FakeReply
	
Code6: ; 6 Forward
	if DebugMode = 0 then debug "RB-> Forward",13,10
	RequestedDistance=RBData*8
	CreateVelocity=CreateSpeed
	CreateRadius=$8000
	if RequestedDistance<>0 then
		if CreateVelocity<>0 then gosub DriveForwardDistance	; CreateVelocity, CreateRadius	
		gosub CreateSensors1
	else
		gosub Start
	endif
	goto OutputToRB
	
Code7: ; 7 Backward
	if DebugMode = 0 then debug "RB-> Backward",13,10
	RequestedDistance=RBData*-8
	CreateVelocity=CreateSpeed*-1
	CreateRadius=$7FFF
	if RequestedDistance<>0 then		
		if CreateVelocity<>0 then gosub DriveBackwardDistance	; CreateVelocity, CreateRadius
		gosub CreateSensors1
	else
		gosub Start
	endif			
	goto OutputToRB
	
Code8:
	goto FakeReply
	
Code9:
	goto FakeReply
	
CodeA:
	goto FakeReply
	
CodeB:
	goto FakeReply
	
CodeC: ; 12 TurnRight
	if DebugMode = 0 then debug "RB-> TurnRight",13,10
	if CreateVelocity>$8000 then
		CreateVelocity=(CreateSpeed/2)*-1
	else
		CreateVelocity=CreateSpeed/2	
	endif
			
	RequestedAngle=RBData*-1
	CreateVelocity=CreateVelocity/2	
	if RBData=0 then
		CreateRadius = $8000 
	else
		CreateRadius = $FFFF 
	endif
	
	if RequestedAngle<>0 then
		if CreateVelocity<>0 then gosub DriveAngle	; CreateVelocity, CreateRadius
	endif

	gosub CreateSensors1
				
	goto OutputToRB
	
CodeD: ; 13 TurnLeft
	if DebugMode = 0 then debug "RB-> TurnLeft",13,10
	if CreateVelocity>$8000 then
		CreateVelocity=(CreateSpeed/2)*-1
	else
		CreateVelocity=CreateSpeed/2	
	endif
	
	RequestedAngle=RBData		
	if RBData=0 then
		CreateRadius = $7FFF
	else
		CreateRadius = $0001 
	endif
	
	if RequestedAngle<>0 then		
		if CreateVelocity<>0 then gosub DriveAngle	; CreateVelocity, CreateRadius
	endif
	
	gosub CreateSensors1
	
	goto OutputToRB
	
CodeE:
	goto FakeReply
	
CodeF:
	goto FakeReply

Code10_1F:
	on RBCommand-$10 goto Code10, Code11, Code12, Code13, Code14, Code15, Code16, Code17, Code18, Code19, Code1A, Code1B, Code1C, Code1D, Code1E, Code1F
	
Code10:
	goto FakeReply
	
Code11:
	goto FakeReply
	
Code12:
	goto FakeReply
	
Code13:
	goto FakeReply
	
Code14:
	goto FakeReply
	
Code15:
	goto FakeReply
	
Code16:
	goto FakeReply
	
Code17:
	goto FakeReply
	
Code18:	; 24 Compass, RBData=0-255
;	HSEROUT ["rCompass="]
;	goto GetReply
	goto FakeReply
	
Code19:
	goto FakeReply
	
Code1A:
	goto FakeReply
	
Code1B:
	goto FakeReply
	
Code1C:
	goto FakeReply
	
Code1D:
	goto FakeReply
	
Code1E:
	goto FakeReply
	
Code1F:
	goto FakeReply
	
Code20_2F:
	on RBCommand-$20 goto Code20, Code21, Code22, Code23, Code24, Code25, Code26, Code27, Code28, Code29, Code2A, Code2B, Code2C, Code2D, Code2E, Code2F
	
Code20:
	goto FakeReply
	
Code21:
	goto FakeReply
	
Code22:
	goto FakeReply
	
Code23:
	goto FakeReply
	
Code24:	; 36 Speed, RBData=0-255 0=fastest 255=slowest
	if DebugMode = 0 then debug "RB-> Speed",13,10
	CreateSpeed=(255-RBData)*2
	if CreateSpeed>500 then CreateSpeed=500
	gosub CreateSensors1
	goto OutputToRB
	
Code25:
	goto FakeReply
	
Code26:
	goto FakeReply
	
Code27:
	goto FakeReply
	
Code28:
	goto FakeReply
	
Code29:
	goto FakeReply
	
Code2A:
	goto FakeReply
	
Code2B:
	goto FakeReply
	
Code2C:
	goto FakeReply
	
Code2D:
	goto FakeReply
	
Code2E:
	goto FakeReply
	
Code2F:
	goto FakeReply

Code30_3F:
	on RBCommand-$30 goto Code30, Code31, Code32, Code33, Code34, Code35, Code36, Code37, Code38, Code39, Code3A, Code3B, Code3C, Code3D, Code3E, Code3F
	
Code30:	; 48 Look Right, RBData=0-255
;	HSEROUT ["rLookRight="]
;	goto GetReply
	goto FakeReply
	
Code31:	; 49 Look Left, RBData=0-255
;	HSEROUT ["rLookLeft="]
;	goto GetReply
	goto FakeReply
	
Code32:
	goto FakeReply
	
Code33:
	goto FakeReply
	
Code34:
	goto FakeReply
	
Code35:
	goto FakeReply
	
Code36:
	goto FakeReply
	
Code37:
	goto FakeReply
	
Code38:
	goto FakeReply
	
Code39:
	goto FakeReply
	
Code3A:
	goto FakeReply
	
Code3B:
	goto FakeReply
	
Code3C:
	goto FakeReply
	
Code3D:
	goto FakeReply
	
Code3E:
	goto FakeReply
	
Code3F:
	goto FakeReply

Code40_4F:
	on RBCommand-$40 goto Code40, Code41, Code42, Code43, Code44, Code45, Code46, Code47, Code48, Code49, Code4A, Code4B, Code4C, Code4D, Code4E, Code4F
	
Code40:
	goto FakeReply
	
Code41:
	goto FakeReply
	
Code42:
	goto FakeReply
	
Code43:
	goto FakeReply
	
Code44:
	goto FakeReply
	
Code45:
	goto FakeReply
	
Code46:
	goto FakeReply
	
Code47:
	goto FakeReply
	
Code48:
	goto FakeReply
	
Code49:
	goto FakeReply
	
Code4A:
	goto FakeReply
	
Code4B:
	goto FakeReply
	
Code4C:
	goto FakeReply
	
Code4D:
	goto FakeReply
	
Code4E:
	goto FakeReply
	
Code4F:
	goto FakeReply

Code50_5F:
	on RBCommand-$50 goto Code50, Code51, Code52, Code53, Code54, Code55, Code56, Code57, Code58, Code59, Code5A, Code5B, Code5C, Code5D, Code5E, Code5F
	
Code50:
	goto FakeReply
	
Code51:
	goto FakeReply
	
Code52:
	goto FakeReply
	
Code53:
	goto FakeReply
	
Code54:
	goto FakeReply
	
Code55:
	goto FakeReply
	
Code56:
	goto FakeReply
	
Code57:
	goto FakeReply
	
Code58:
	goto FakeReply
	
Code59:
	goto FakeReply
	
Code5A:
	goto FakeReply
	
Code5B:
	goto FakeReply
	
Code5C:
	goto FakeReply
	
Code5D:
	goto FakeReply
	
Code5E:
	goto FakeReply
	
Code5F:
	goto FakeReply

Code60_6F:
	on RBCommand-$60 goto Code60, Code61, Code62, Code63, Code64, Code65, Code66, Code67, Code68, Code69, Code6A, Code6B, Code6C, Code6D, Code6E, Code6F
	
Code60:	; 96 Beacon, RBData=0-255
	if DebugMode = 0 then debug "RB-> Beacon",13,10
	CreateColor=RBData
	gosub CreateSensors17
	RROSbufferIn[3]=0
	RROSbufferIn[4]=0
	if CreateIR.0=0 then
		if CreateColor=4 and CreateIR.3=1 then 	RROSbufferIn[4]=1 ; red
		if CreateColor=2 and CreateIR.2=1 then 	RROSbufferIn[4]=1 ; green
		if CreateColor=1 and CreateIR.1=1 then 	RROSbufferIn[4]=1 ; blue (force)		
	endif
	goto OutputToRB
	
Code61:
	goto FakeReply
	
Code62:
	goto FakeReply
	
Code63:
	goto FakeReply
	
Code64:
	goto FakeReply
	
Code65:
	goto FakeReply
	
Code66:
	goto FakeReply
	
Code67:
	goto FakeReply
	
Code68:
	goto FakeReply
	
Code69:
	goto FakeReply
	
Code6A:
	goto FakeReply
	
Code6B:
	goto FakeReply
	
Code6C:	; 108 Charge Level, RBData=0-255
	if DebugMode = 0 then debug "RB-> ChargeLevel",13,10
;	gosub CreateSensors25
;	PresentCharge=MSByte*256+LSByte
;	gosub CreateSensors26
;	MaxCharge=(MSByte*256+LSByte)/100
;	Percent=PresentCharge/MaxCharge

	gosub CreateSensors22
	PresentVolts=(MSByte*256+LSByte)
	Percent=(PresentVolts-MinCharge)/((FullCharge-MinCharge)/100)
	gosub CreateSensors1
	if PresentVolts<MinCharge then
		RROSbufferIn[4]=0
	else
		RROSbufferIn[4]=Percent		
	endif
	goto OutputToRB	
	
Code6D:
	goto FakeReply
	
Code6E:	; RBData=0-255
;	HSEROUT ["CalibServoDrive="]
	goto FakeReply	
	
Code6F:	; RBData=0-255
;	HSEROUT ["CalibComapass="]
	goto FakeReply	

Code70_7F:
	on RBCommand-$70 goto Code70, Code71, Code72, Code73, Code74, Code75, Code76, Code77, Code78, Code79, Code7A, Code7B, Code7C, Code7D, Code7E, Code7F
	
Code70:	; RBData=0-255
;	HSEROUT ["SetSenseInvMask="]
	goto FakeReply
	
Code71:	; RBData=0-255
;	HSEROUT ["SetTurretOffset="]
	goto FakeReply
	
Code72:	; RBData=0-255
;	HSEROUT ["SetRotationTime="]
	goto FakeReply
	
Code73:	; RBData=0-255
;	HSEROUT ["SetMoveTime="]
	goto FakeReply
	
Code74:	; RBData=0-255
;	HSEROUT ["SetLeftStopOffset="]
	goto FakeReply
	
Code75:	; RBData=0-255
;	HSEROUT ["SetRightStopOffset"]
	goto FakeReply
	
Code76:	; RBData=0-255
;	HSEROUT ["SetTurnStyle="]
	goto FakeReply
	
Code77:	; 119 SetSpeed, RBData=0-100
	if DebugMode = 0 then debug "RB-> SetSpeed",13,10
	CreateSpeed=RBData*5
	gosub CreateSensors1
	goto OutputToRB
	
Code78:	; RBData=0-255
;	HSEROUT ["SetCCDivisor="]
	goto FakeReply
	
Code79:	; 121 SetMotorRamp, RBData=1-255
;	if DebugMode = 0 then debug "RB-> SetMotorRamp",13,10
;	RampDelta=RBData
;	gosub CreateSensors1
;	goto OutputToRB
	goto FakeReply
	
Code7A:	; RBData=0-255
;	HSEROUT ["SetBumpDistance="]
	goto FakeReply
	
Code7B:	; RBData=0-255
;	HSEROUT ["SetProxDistance="]
	goto FakeReply
	
Code7C:	; RBData=0-255
;	HSEROUT ["SetExpanTimeout="]
	goto FakeReply
	
Code7D:	; RBData=0-255
;	HSEROUT ["SetRobotAngle="]
	goto FakeReply
	
Code7E:	; RBData=0-255
;	HSEROUT ["SetBeaconTIme="]
	goto FakeReply
	
Code7F:	; RBData=0-255
;	HSEROUT ["SetClicksPerDiam="]
	goto FakeReply

Code80_8F:
	on RBCommand-$80 goto Code80, Code81, Code82, Code83, Code84, Code85, Code86, Code87, Code88, Code89, Code8A, Code8B, Code8C, Code8D, Code8E, Code8F
	
Code80:	; RBData=0-255
;	HSEROUT ["SetClicksPer90="]
	goto FakeReply
	
Code81:	; RBData=0-255
;	HSEROUT ["rPen="]
	goto FakeReply
	
Code82:	; RBData=0-255
;	HSEROUT ["TurnToHalfAngle="]
	goto FakeReply
	
Code83:	; 131 Play Sound, RBData=0-255
	if DebugMode = 0 then debug "RB-> Play Sound",13,10
	Parameter=RBData
	
	x=Parameter & %00011111
	gosub ConvertNote
	NoteNumber[0]=x
	
	x=Parameter/32
	gosub ConvertDuration
	NoteDuration[0]=x
	
	SongNumber=0
	NumberOfNotes=1
	gosub CreateSong
	gosub CreatePlay
	gosub CreateSensors37	
	gosub CreateSensors1
	goto OutputToRB
	
Code84:	; RBData=0-255
;	HSEROUT ["TestCommand="]
	goto FakeReply
	
Code85:
	goto FakeReply
	
Code86:	; RBData=0-255
;	HSEROUT ["EnableMotors="]
	goto FakeReply
	
Code87:	; RBData=0-255
;	HSEROUT ["EnableCounters="]
	goto FakeReply
	
Code88:	; RBData=0-255
;	HSEROUT ["SetSlowDownSpeed="]
	goto FakeReply
	
Code89:	; RBData=0-255
;	HSEROUT ["MuteBeaconDet="]
	goto FakeReply
	
Code8A:	; RBData=0-255
;	HSEROUT ["MotorSetup="]
	goto FakeReply
	
Code8B:	; RBData=0-255
;	HSEROUT ["SensorSetup="]
	goto FakeReply
	
Code8C:	; RBData=0-255
;	HSEROUT ["SetReducForRight="]
	goto FakeReply
	
Code8D:	; RBData=0-255
;	HSEROUT ["SetReducBackRight="]
	goto FakeReply
	
Code8E:	; RBData=0-255
;	HSEROUT ["SetReducForLeft="]
	goto FakeReply
	
Code8F:	; RBData=0-255
;	HSEROUT ["SetReducBackLeft="]
	goto FakeReply

Code90_9F:
	on RBCommand-$90 goto Code90, Code91, Code92, Code93, Code94, Code95, Code96, Code97, Code98, Code99, Code9A, Code9B, Code9C, Code9D, Code9E, Code9F
	
Code90:	; RBData=0-255
;	HSEROUT ["SetRROSTimeout="]
	goto FakeReply
	
Code91:	; RBData=0-255
;	HSEROUT ["SetCountDivisor="]
	goto FakeReply
	
Code92:	; RBData=0-255
;	HSEROUT ["SetDriveServWidth="]
	goto FakeReply
	
Code93:	; RBData=0-255
;	HSEROUT ["FindBeacon="]
	goto FakeReply
	
Code94:	; RBData=0-255
;	HSEROUT ["SetBeaconDelay="]
	goto GetReply
	
Code95:	; RBData=0-255
;	HSEROUT ["SetSteerServoWidth="]
	goto FakeReply
	
Code96:	; RBData=0-255
;	HSEROUT ["SetDriveServoDir="]
	goto FakeReply
		
Code97:	; RBData=0-255
;	HSEROUT ["SetSteerServoDir="]
	goto FakeReply
	
Code98:	; RBData=0-255
;	HSEROUT ["SetRobotDiameter="]
	goto FakeReply
	
Code99:	; RBData=0-255
;	HSEROUT ["SetServoTurretWidth="]
	goto FakeReply
	
Code9A:	; RBData=0-255
;	HSEROUT ["SetTurnToMax="]
	goto FakeReply
	
Code9B:	; RBData=0-255
;	HSEROUT ["SetTurnToMin="]
	goto FakeReply
	
Code9C:	; RBData=0-255
;	HSEROUT ["SetSlowDown2="]
	goto FakeReply
	
Code9D:	; RBData=0-255
;	HSEROUT ["SetPCBCalibrate="]
	goto FakeReply
	
Code9E:
	goto FakeReply
	
Code9F:	; 159 ArcTurn, RBData=0-255 where Bits7->0=forward, 1=backward, Bit6->0=right, 1=left, Bits5:0=percent reduction for slower wheel
	WheelRatio=(RBData & %00111111) * 2		
	if WheelRatio<50 then
		CreateRadius=(WheelRatio*5)+127
	else
		if WheelRatio<100 then
			CreateRadius=((WheelRatio-50)*33)+381
		else
			CreateRadius=32767
		endif
	endif

	if DebugMode = 0 then debug "CreateRadius ", HEX CreateRadius.HIGHBYTE, " ", HEX CreateRadius.LOWBYTE,13,10	
	CreateVelocity=CreateSpeed  		; gosub RampUpDown
	
	if RBData.7=0 then goto Cod9F_Forward

Cod9F_Backward:
	CreateVelocity=(CreateVelocity*-1)-1
	if RBData.6=0 then goto Cod9F_Backward_Right
	
Cod9F_Backward_Left:
	if DebugMode = 0 then debug "RB->ArcTurn_BL",13,10
	goto Cod9F_Continue
		
Cod9F_Backward_Right:
	if DebugMode = 0 then debug "RB->ArcTurn_BR",13,10
	CreateRadius=(CreateRadius*-1)-1	
	goto Cod9F_Continue

Cod9F_Forward:		
	if RBData.6=0 then goto Cod9F_Forward_Right
	
Cod9F_Forward_Left:
	if DebugMode = 0 then debug "RB->ArcTurn_FL",13,10
	goto Cod9F_Continue

Cod9F_Forward_Right:
	if DebugMode = 0 then debug "RB->ArcTurn_FR",13,10
	CreateRadius=(CreateRadius*-1)-1	
	goto Cod9F_Continue
	
Cod9F_Continue:
	gosub Drive
	
	goto OutputToRB	
	
RampUpDown:
	if CreateVelocity<CreateSpeed then	
		CreateVelocity=CreateVelocity+RampIncrement
		if CreateVelocity>CreateSpeed then CreateVelocity=CreateSpeed
	else
		if CreateVelocity>CreateSpeed then	
			CreateVelocity=CreateVelocity-RampIncrement
			if CreateVelocity<CreateSpeed then CreateVelocity=CreateSpeed
			if CreateVelocity<0 then CreateVelocity=CreateSpeed	
		endif
	endif
	if DebugMode = 0 then debug "Ramp ", HEX CreateSpeed.HIGHBYTE, " ", HEX CreateSpeed.LOWBYTE, " ", HEX CreateVelocity.HIGHBYTE, " ", HEX CreateVelocity.LOWBYTE,13,10	
	return	
	
	
	
	
	
	
	if RBData.7=0 then
		CreateVelocity=CreateSpeed
		if CreateVelocity>500 then CreateVelocity=500
	else
		CreateVelocity=$FFFF - CreateSpeed	
		if CreateVelocity>($FFFF-500) then CreateRadius=$FFFF-500		
	endif

	Percent=(RBData & %00111111) * 2
		
	if RBData.6=0 then 					; 0=$7FFF (infinite radius) 100=128 (1 radius)
		if Percent=100 then
			CreateRadius=$8000
		else
			CreateRadius=$FFFF - (Percent * 20) ; RBData.6=0 right
		endif
		if CreateRadius<$8000 then CreateRadius=$8000
	else
		if Percent=100 then
			CreateRadius=$7FFF
		else
			CreateRadius= 1 + (Percent * 20) ; RBData.6=1 left
		endif	
		if CreateRadius>$7FFF then CreateRadius=$7FFF
	endif
	
	gosub Drive
	
	goto OutputToRB

CodeA0_AF:
	on RBCommand-$A0 goto CodeA0, CodeA1, CodeA2, CodeA3, CodeA4, CodeA5, CodeA6, CodeA7, CodeA8, CodeA9, CodeAA, CodeAB, CodeAC, CodeAD, CodeAE, CodeAF
	
CodeA0:
	goto FakeReply
	
CodeA1:
	goto FakeReply
	
CodeA2:
	goto FakeReply
	
CodeA3:
	goto FakeReply
	
CodeA4:
	goto FakeReply
	
CodeA5:
	goto FakeReply
	
CodeA6:
	goto FakeReply
	
CodeA7:
	goto FakeReply
	
CodeA8:
	goto FakeReply
	
CodeA9:
	goto FakeReply
	
CodeAA:
	goto FakeReply
	
CodeAB:
	goto FakeReply
	
CodeAC:
	goto FakeReply
	
CodeAD:
	goto FakeReply
	
CodeAE:
	goto FakeReply
	
CodeAF:
	goto FakeReply

CodeB0_BF:
	on RBCommand-$B0 goto CodeB0, CodeB1, CodeB2, CodeB3, CodeB4, CodeB5, CodeB6, CodeB7, CodeB8, CodeB9, CodeBA, CodeBB, CodeBC, CodeBD, CodeBE, CodeBF
	
CodeB0:
	goto FakeReply
	
CodeB1:
	goto FakeReply
	
CodeB2:
	goto FakeReply
	
CodeB3:
	goto FakeReply
	
CodeB4:
	goto FakeReply
	
CodeB5:
	goto FakeReply
	
CodeB6:
	goto FakeReply
	
CodeB7:
	goto FakeReply
	
CodeB8:
	goto FakeReply
	
CodeB9:
	goto FakeReply
	
CodeBA:
	goto FakeReply
	
CodeBB:
	goto FakeReply
	
CodeBC:
	goto FakeReply
	
CodeBD:
	goto FakeReply
	
CodeBE:
	goto FakeReply
	
CodeBF:
	goto FakeReply

CodeC0_CF:
	on RBCommand-$C0 goto CodeC0, CodeC1, CodeC2, CodeC3, CodeC4, CodeC5, CodeC6, CodeC7, CodeC8, CodeC9, CodeCA, CodeCB, CodeCC, CodeCD, CodeCE, CodeCF
	
CodeC0:	; 192 Range Right, RBData=0-255
;	HSEROUT ["rRangeRight="]
	goto FakeReply
	
CodeC1:	; 193 Range Left, RBData=0-255
;	HSEROUT ["rRangeLeft="]
	goto FakeReply
	
CodeC2:
	goto FakeReply
	
CodeC3:
	goto FakeReply
	
CodeC4:
	goto FakeReply
	
CodeC5:
	goto FakeReply
	
CodeC6:
	goto FakeReply
	
CodeC7:
	goto FakeReply
	
CodeC8:	; RBData=0-255
;	HSEROUT ["ExpansionSetup="]
	goto FakeReply
	
CodeC9:	; RBData=0-255
;	HSEROUT ["BecomeArmExpansion="]
	goto FakeReply
	
CodeCA:	; RBData=0-255
;	HSEROUT ["ReadExpansionSensors="]
	goto FakeReply
	
CodeCB:	; RBData=0-255
;	HSEROUT ["ReadArmSensors="]
	goto FakeReply
	
CodeCC:
	goto FakeReply
	
CodeCD:
	goto FakeReply
	
CodeCE:
	goto FakeReply
	
CodeCF:
	goto FakeReply

CodeD0_DF:
	on RBCommand-$D0 goto CodeD0, CodeD1, CodeD2, CodeD3, CodeD4, CodeD5, CodeD6, CodeD7, CodeD8, CodeD9, CodeDA, CodeDB, CodeDC, CodeDD, CodeDE, CodeDF
	
CodeD0:
	goto FakeReply
	
CodeD1:
	goto FakeReply
	
CodeD2:	; 210 Create Custom Command, RBData=0-255
	if RBData=0 then					; Cover and Dock
		if DebugMode = 0 then debug "->Create Cover and Dock", 13, 10
		HSEROUT [136,1]
		gosub CreateSensors1		
	endif
	if RBData=1 then					; Charging State
		if DebugMode = 0 then debug "->Create Sensors21", 13, 10
		gosub CreateSensors1
		gosub CreateSensors21
		RROSbufferIn[4]=LSByte
	endif
	if RBData=2 then					; IO Mode
		if DebugMode = 0 then debug "->Create Sensors35", 13, 10		
		gosub CreateSensors1
		gosub CreateSensors35
		RROSbufferIn[4]=LSByte
	endif	
	goto OutputToRB
	
CodeD3:
	goto FakeReply
	
CodeD4:
	goto FakeReply
	
CodeD5:
	goto FakeReply
	
CodeD6:
	goto FakeReply
	
CodeD7:
	goto FakeReply
	
CodeD8:
	goto FakeReply
	
CodeD9:
	goto FakeReply
	
CodeDA:
	goto FakeReply
	
CodeDB:
	goto FakeReply
	
CodeDC:
	goto FakeReply
	
CodeDD:
	goto FakeReply
	
CodeDE:
	goto FakeReply
	
CodeDF:
	goto FakeReply

CodeE0_EF:
	on RBCommand-$E0 goto CodeE0, CodeE1, CodeE2, CodeE3, CodeE4, CodeE5, CodeE6, CodeE7, CodeE8, CodeE9, CodeEA, CodeeB, CodeEC, CodeED, CodeEE, CodeEF
	
CodeE0:
	goto FakeReply
	
CodeE1:
	goto FakeReply
	
CodeE2:
	goto FakeReply
	
CodeE3:
	goto FakeReply
	
CodeE4:
	goto FakeReply
	
CodeE5:
	goto FakeReply
	
CodeE6:	; RBData=0-255
;	HSEROUT ["SetNAVBeacons="]
	goto FakeReply
	
CodeE7:	; RBData=0-255
;	HSEROUT ["ReadNAVAngles="]
	goto FakeReply
	
CodeE8:	; RBData=0-255
;	HSEROUT ["ReadArmAnalogs="]
	goto FakeReply
	
CodeE9:	; RBData=0-255
;	HSEROUT ["SetServoNumber="]
	goto FakeReply
	
CodeEA:	; RBData=0-255
;	HSEROUT ["SetServoPosition="]
	goto FakeReply
	
CodeEB:	; RBData=0-255
;	HSEROUT ["SetServoMin="]
	goto FakeReply
	
CodeEC:	; RBData=0-255
;	HSEROUT ["SetServoSpeed="]
	goto FakeReply
	
CodeED:	; RBData=0-255
;	HSEROUT ["SetServoMax="]
	goto FakeReply
	
CodeEE:	; RBData=0-255
;	HSEROUT ["EnableServos="]
	goto FakeReply
	
CodeEF:	; RBData=0-255
;	HSEROUT ["GPSSetup="]
	goto FakeReply

CodeF0_FF:
	on RBCommand-$F0 goto CodeF0, CodeF1, CodeF2, CodeF3, CodeF4, CodeF5, CodeF6, CodeF7, CodeF8, CodeF9, CodeFA, CodeFB, CodeFC, CodeFD, CodeFE, CodeFF
	
CodeF0:	; RBData=0-255
;	HSEROUT ["SetGPSHighX="]
	goto FakeReply
	
CodeF1:	; RBData=0-255
;	HSEROUT ["SetGPSLowX="]
	goto FakeReply
	
CodeF2:	; RBData=0-255
;	HSEROUT ["SetGPSHighY="]
	goto FakeReply
	
CodeF3:	; RBData=0-255
;	HSEROUT ["SetGPSLowY="]
	goto FakeReply
	
CodeF4:	; RBData=0-255
;	HSEROUT ["ExtSensorSetup="]
	goto FakeReply
	
CodeF5:	; RBData=0-255
;	HSEROUT ["LookSetup="]
	goto FakeReply
	
CodeF6:	; RBData=0-255
;	HSEROUT ["PenSetup="]
	goto FakeReply
	
CodeF7:	; RBData=0-255
;	HSEROUT ["SetArmOutputs="]
	goto FakeReply
	
CodeF8:	; RBData=0-255
;	HSEROUT ["SetNavServoMinHigh="]
	goto FakeReply
	
CodeF9:	; RBData=0-255
;	HSEROUT ["SetNavServoMinLow="]
	goto FakeReply
	
CodeFA:	; RBData=0-255
;	HSEROUT ["SetNavServoMaxLow="]
	goto FakeReply
	
CodeFB:	; RBData=0-255
;	HSEROUT ["SetNavServoMaxHigh="]
	goto FakeReply
	
CodeFC:
	goto FakeReply
	
CodeFD:
	goto FakeReply
	
CodeFE:
	goto FakeReply
	
CodeFF:
	goto FakeReply

Drive:
	if DebugMode = 0 then debug "->Create Drive",  HEX CreateVelocity.Byte1, " ", HEX CreateVelocity.Byte0, " ", HEX CreateRadius.Byte1, " ", HEX CreateRadius.Byte0, 13, 10
	HSEROUT [137,CreateVelocity.Byte1,CreateVelocity.Byte0,CreateRadius.Byte1,CreateRadius.Byte0]
	Gosub CreateSensors1
	return
	
DriveForwardDistance:
	if DebugMode = 0 then debug "->Create Drive Forward Distance ", HEX CreateVelocity.Byte1, " ", HEX CreateVelocity.Byte0, " ", HEX CreateRadius.Byte1, " ", HEX CreateRadius.Byte0, 13, 10
	HSEROUT [137,CreateVelocity.Byte1,CreateVelocity.Byte0,CreateRadius.Byte1,CreateRadius.Byte0]
	CreateDistance=0
		
DriveForwardDistanceChk:
	pause 15
	if DebugMode = 0 then debug "Requested=",HEX RequestedDistance.HIGHBYTE, HEX RequestedDistance.LOWBYTE, 13, 10
	Gosub CreateSensors19
	CreateDistance=CreateDistance+(MSByte*256)+LSByte
	if DebugMode = 0 then debug "Actual=",HEX CreateDistance.HIGHBYTE, HEX CreateDistance.LOWBYTE, 13, 10	
	if CreateDistance<RequestedDistance then goto DriveForwardDistanceChk
	return
	
DriveBackwardDistance:
	if DebugMode = 0 then debug "->Create Drive Backward Distance",  HEX CreateVelocity.Byte1, " ", HEX CreateVelocity.Byte0, " ", HEX CreateRadius.Byte1, " ", HEX CreateRadius.Byte0, 13, 10
	HSEROUT [137,CreateVelocity.Byte1,CreateVelocity.Byte0,CreateRadius.Byte1,CreateRadius.Byte0]
	CreateDistance=$FFFF
	
DriveBackwardDistanceChk:
	pause 15
	if DebugMode = 0 then debug "Requested=",HEX RequestedDistance.HIGHBYTE, HEX RequestedDistance.LOWBYTE, 13, 10
	Gosub CreateSensors19
	CreateDistance=CreateDistance+(MSByte*256)+LSByte
	if DebugMode = 0 then debug "Actual=",HEX CreateDistance.HIGHBYTE, HEX CreateDistance.LOWBYTE, 13, 10	
	if CreateDistance>RequestedDistance then goto DriveBackwardDistanceChk
	return	

DriveAngle:
	if DebugMode = 0 then debug "->Create Drive ",HEX CreateVelocity.Byte1, " ", HEX CreateVelocity.Byte0, " ", HEX CreateRadius.Byte1, " ", HEX CreateRadius.Byte0, 13, 10
	HSEROUT [137,CreateVelocity.Byte1,CreateVelocity.Byte0,CreateRadius.Byte1,CreateRadius.Byte0]
	if RequestedAngle>$8000	then	; CW
		if DebugMode = 0 then debug "Turning CW (decresaing angle)", 13, 10
		CreateAngle=$FFFF
		goto DriveDecrementAngle
	else							; CCW
		if DebugMode = 0 then debug "Turning CCW (increasing angle)", 13, 10
		CreateAngle=$0000
		goto DriveIncrementAngle		
	endif
	
DriveDecrementAngle:
	pause 15
	if DebugMode = 0 then debug "Requested=",HEX RequestedAngle.HIGHBYTE, HEX RequestedAngle.LOWBYTE, 13, 10
	Gosub CreateSensors20
	CreateAngle=CreateAngle+(MSBYte*256)+LSByte
	if DebugMode = 0 then debug "Actual=",HEX CreateAngle.HIGHBYTE, HEX CreateAngle.LOWBYTE, 13, 10
	if CreateAngle>RequestedAngle then goto DriveDecrementAngle
	return

DriveIncrementAngle:
	pause 15
	if DebugMode = 0 then debug "Requested=",HEX RequestedAngle.HIGHBYTE, HEX RequestedAngle.LOWBYTE, 13, 10
	Gosub CreateSensors20
	CreateAngle=CreateAngle+(MSBYte*256)+LSByte
	if DebugMode = 0 then debug "Actual=",HEX CreateAngle.HIGHBYTE, HEX CreateAngle.LOWBYTE, 13, 10
	if CreateAngle<RequestedAngle then goto DriveIncrementAngle
	return
	
Start:
	if DebugMode = 0 then debug "->Create Passive", 13, 10
	HSEROUT [128]
	HSERIN 10, Start_1,[Dummy, Dummy, Dummy, Dummy, Dummy]
	
Start_1:		
	Gosub CreateSensors35
	if LSByte=255 then return
	if LSByte <> 1 then goto Start
	
	if DebugMode = 0 then debug "->Create Full", 13, 10
	HSEROUT [132]	
	Gosub CreateSensors35
	if LSByte=255 then return	
	if LSByte <> 3 then goto Start	

	Gosub CreateSensors19	
	Gosub CreateSensors20	
	Gosub CreateSensors1	
	return
	
ConvertNote:
	on x/16 gosub ConvertNote0_F, ConvertNote10_1F
	
ConvertNote0_F:
	lookup x,[0, 60, 62, 64, 65, 67, 69, 71, 72, 74, 76, 77, 79, 81, 83, 84],x
	goto ConvertNoteExit
	
ConvertNote10_1F:
	lookup x-16,[86, 88, 89, 91, 93, 95, 96, 98, 100, 101, 103, 105, 107, 108, 110, 112],x
	goto ConvertNoteExit	

ConvertNoteExit:
	return
	
ConvertDuration:
	lookup x,[2, 4, 6, 12, 16, 20, 24, 32],x
	return
	
CreateSensors1:							; Sensors 7-16
	if DebugMode = 0 then debug "->Create Sensors1", 13, 10
	HSEROUT [142,1]
	HSERIN 1000, CreateSensors1TimeOut, [CreateBump,CreateWall,CreateLCliff, CreateFLCliff, CreateFRCliff, CreateRCliff, CreateVWall, CreateOverC, CreateDummy, CreateDummy]
	if DebugMode = 0 then debug "Create-> ", HEX CreateBump, " ", HEX CreateWall, " ", HEX CreateLCliff, " ", HEX CreateFLCliff, " ", HEX CreateFRCliff, " ", HEX CreateRCliff, " ", HEX CreateVWall, " ", HEX CreateOverC, " ", HEX CreateDummy, " ", HEX CreateDummy, 13, 10
	goto CreateSensors1Continue
	
CreateSensors1TimeOut:
	debug "Timeout", 13, 10	
	RROSbufferIn[0]=0
	RROSbufferIn[1]=0
	RROSbufferIn[2]=0
	RROSbufferIn[3]=0
	RROSbufferIn[4]=0
	return				
	
CreateSensors1Continue:	
	RROSbufferIn[0]=0
	if CreateBump.0=1 then RROSbufferIn[0]=RROSbufferIn[0]|2 ; right
	if CreateBump.1=1 then RROSbufferIn[0]=RROSbufferIn[0]|8 ; left	
	if CreateBump.2=1 then RROSbufferIn[0]=RROSbufferIn[0]|2 ; right	
	if CreateBump.3=1 then RROSbufferIn[0]=RROSbufferIn[0]|8 ; left
	if CreateBump.4=1 then RROSbufferIn[0]=RROSbufferIn[0]|4 ; front
	RROSbufferIn[1]=0
	if CreateWall=1 then RROSbufferIn[1]=RROSbufferIn[1]|1 ; right	
	if CreateLCliff=1 then RROSbufferIn[1]=RROSbufferIn[1]|16 ; left	
	if CreateFLCliff=1 then RROSbufferIn[1]=RROSbufferIn[1]|8 ; leftfront	
	if CreateRCliff=1 then RROSbufferIn[1]=RROSbufferIn[1]|1 ; right	
	if CreateFRCliff=1 then RROSbufferIn[1]=RROSbufferIn[1]|2 ; rightfront				
	if CreateVWall=1 then RROSbufferIn[1]=RROSbufferIn[1]|4 ; front	
	RROSbufferIn[2]=0
	RROSbufferIn[3]=0
	RROSbufferIn[4]=0		
	return

CreateSensors17:						; IR Byte
	if DebugMode = 0 then debug "->Create Sensors17", 13, 10	
	HSEROUT [142,17]
	HSERIN 1000, CreateSensors17TimeOut, [CreateIR]
	goto CreateSensors17Continue
	
CreateSensors17TimeOut:
	debug "Timeout", 13, 10
	CreateIR=0	
			
CreateSensors17Continue:
	if DebugMode = 0 then debug "Create-> ", HEX CreateIR, 13, 10	
	Gosub CreateSensors1	
	return	
	
CreateSensors19:						; Distance	
	if DebugMode = 0 then debug "->Create Sensors19", 13, 10
	HSEROUT [142,19]
	HSERIN 1000, CreateSensors19TimeOut, [MSByte,LSByte]
	goto CreateSensors19Continue
	
CreateSensors19TimeOut:
	debug "Timeout", 13, 10	
	MSByte=0
	LSByte=0
		
CreateSensors19Continue:
	if DebugMode = 0 then debug "Create-> ", HEX MSByte, " " ,HEX LSByte, 13, 10	
	return
	
CreateSensors20:						; Angle
	if DebugMode = 0 then debug "->Create Sensors20", 13, 10
	HSEROUT [142,20]
	HSERIN 1000, CreateSensors20TimeOut, [MSByte,LSByte]
	goto CreateSensors20Continue
	
CreateSensors20TimeOut:
	debug "Timeout", 13, 10	
	MSByte=0
	LSByte=0
		
CreateSensors20Continue:
	if DebugMode = 0 then debug "Create-> ", HEX MSByte, " " ,HEX LSByte, 13, 10	
	return

CreateSensors21:						; Charging State
	if DebugMode = 0 then debug "->Create Sensors21", 13, 10
	HSEROUT [142,21]
	HSERIN 1000, CreateSensors21TimeOut, [LSByte]
	goto CreateSensors21Continue
	
CreateSensors21TimeOut:
	LSByte=255
	debug "Timeout", 13, 10	
	
CreateSensors21Continue:
	if DebugMode = 0 then debug "Create-> ", HEX LSByte, 13, 10	
	return
		
CreateSensors22:						; Voltage
	if DebugMode = 0 then debug "->Create Sensors22", 13, 10
	HSEROUT [142,22]
	HSERIN 1000, CreateSensors22TimeOut, [MSByte,LSByte]
	goto CreateSensors22Continue
	
CreateSensors22TimeOut:
	debug "Timeout", 13, 10	
	MSByte=0
	LSByte=0
	
CreateSensors22Continue:		
	if DebugMode = 0 then debug "Create-> ", HEX MSByte, " " ,HEX LSByte, 13, 10	
	return
		
CreateSensors25:						; Battery Charge
	if DebugMode = 0 then debug "->Create Sensors25", 13, 10
	HSEROUT [142,25]
	HSERIN 1000, CreateSensors25TimeOut, [MSByte,LSByte]
	goto CreateSensors25Continue
	
CreateSensors25TimeOut:
	debug "Timeout", 13, 10	
	MSByte=0
	LSByte=0
		
CreateSensors25Continue:
	if DebugMode = 0 then debug "Create-> ", HEX MSByte, " " ,HEX LSByte, 13, 10	
	return
	
CreateSensors26:						; Battery Capacity
	if DebugMode = 0 then debug "->Create Sensors26", 13, 10
	HSEROUT [142,26]
	HSERIN 1000, CreateSensors26TimeOut, [MSByte,LSByte]
	goto CreateSensors26Continue
	
CreateSensors26TimeOut:
	debug "Timeout", 13, 10	
	MSByte=0
	LSByte=0
		
CreateSensors26Continue:
	if DebugMode = 0 then debug "Create-> ", HEX MSByte, " " ,HEX LSByte, 13, 10	
	return
	
CreateSensors35:						; OI Mode
	if DebugMode = 0 then debug "->Create Sensors35", 13, 10
	HSEROUT [142,35]
	HSERIN 1000, CreateSensors35TimeOut, [LSByte]
	goto CreateSensors35Continue
	
CreateSensors35TimeOut:
	LSByte=255
	debug "Timeout", 13, 10	
	
CreateSensors35Continue:
	if DebugMode = 0 then debug "Create-> ", HEX LSByte, 13, 10	
	return	
	
CreateSensors37:						; Song Playing
	pause 15
	if DebugMode = 0 then debug "->Create Sensor37 Song Playing", 13, 10
	HSEROUT [142,37]
	HSERIN 1000, CreateSensors37TimeOut, [LSByte]
	if LSByte=1 then goto CreateSensors37
	goto CreateSensors37Continue
	
CreateSensors37TimeOut:
	debug "Timeout", 13, 10	
	LSByte=255
		
CreateSensors37Continue:
	if DebugMode = 0 then debug "Create-> ", HEX LSByte, 13, 10	
	return	
	
CreateSong:
	if DebugMode = 0 then debug "->Create Song ", HEX SongNumber, " ", HEX NumberOfNotes
	for x=0 to NumberOfNotes-1
		if DebugMode = 0 then debug " ", HEX NoteNumber[x], " ", HEX NoteDuration[x]
	next x
	if DebugMode = 0 then debug 13,10
	HSEROUT [140,SongNumber,NumberOfNotes]
	for x=0 to NumberOfNotes-1
		HSEROUT [NoteNumber[x],NoteDuration[x]]
	next x
	return
	
CreatePlay:
	if DebugMode = 0 then debug "->Create Play ", HEX SongNumber, 13, 10
	HSEROUT [141, SongNumber]
	return							