' cdc_desc.bas
' USB descriptors for a CDC device
' 9/16/08 microEngineering Labs, Inc.

'USBMEMORYADDRESS Con	$400	' USB RAM starts here (set in device header file)
USBMEMORYSIZE	Con	256	' USB RAM size in bytes
USBReservedMemory Var Byte[USBMEMORYSIZE] USBMEMORYADDRESS	' Reserve memory used by USB assembler code

	Goto	cdc_desc_end	' Skip over all of the USB assembler code

	Asm

#define USB_EP0_BUFF_SIZE	8	; 8, 16, 32, or 64
#define USB_MAX_NUM_INT		2
#define USB_MAX_EP_NUMBER	3
#define NUM_CONFIGURATIONS	1
#define NUM_INTERFACES		2

#define UCFG_VAL	USB_PULLUP_ENABLE|USB_INTERNAL_TRANSCEIVER|USB_FULL_SPEED|USB_PING_PONG__NO_PING_PONG
;#define UCFG_VAL	USB_PULLUP_ENABLE|USB_INTERNAL_TRANSCEIVER|USB_LOW_SPEED|USB_PING_PONG__NO_PING_PONG

;#define USE_SELF_POWER_SENSE_IO
;#define USE_USB_BUS_SENSE_IO

#define USB_POLLING

; CDC
; Endpoints Allocation
#define CDC_COMM_INTF_ID	0x00
#define CDC_COMM_EP		2
#define CDC_COMM_IN_EP_SIZE	8

#define CDC_DATA_INTF_ID	0x01
#define CDC_DATA_EP		3
#define CDC_DATA_OUT_EP_SIZE	64
#define CDC_DATA_IN_EP_SIZE	64

	include	"usb_cdc.asm"	; Include rest of USB files, starting with CDC class code

; ******************************************************************
; This table is polled by the host immediately after USB Reset has been released.
; This table defines the maximum packet size EP0 can take.
; See section 9.6.1 of the Rev 1.0 USB specification.
; These fields are application DEPENDENT. Modify these to meet
; your specifications.
; ******************************************************************
DeviceDescriptor
	retlw	(EndDeviceDescriptor-DeviceDescriptor)/2	; Size of this descriptor in bytes
	retlw	USB_DESCRIPTOR_DEVICE	; DEVICE descriptor type
	retlw	0x10		; USB Spec Release Number in BCD format - 1.10 (low byte)
	retlw	0x01		; (high byte)
	retlw	CDC_DEVICE	; Class Code
	retlw	0x00		; Subclass code
	retlw	0x00		; Protocol code
	retlw	USB_EP0_BUFF_SIZE	; Max packet size for EP0
	retlw	0xD8		; Vendor ID - 0x04D8 is Microchip Vendor ID (low byte)
	retlw	0x04		; (high byte)
	retlw	0x0A		; Product ID: CDC RS-232 Emulation Demo (low byte)
	retlw	0x00		; (high byte)
	retlw	0x01		; Device release number in BCD format (low byte)
	retlw	0x00		; (high byte)
	retlw	0x01		; Manufacturer string index
	retlw	0x02		; Product string index
	retlw	0x00		; Device serial number string index
	retlw	NUM_CONFIGURATIONS	; Number of possible configurations
EndDeviceDescriptor

; ******************************************************************
; This table is retrieved by the host after the address has been set.
; This table defines the configurations available for the device.
; See section 9.6.2 of the Rev 1.0 USB specification (page 184).
; These fields are application DEPENDENT. 
; Modify these to meet your specifications.
; ******************************************************************
; Configuration pointer table
USB_CD_Ptr
Configs
	db	low Config1, high Config1
	db	upper Config1, 0

; Configuration Descriptor
Config1
	retlw	(Interface1-Config1)/2	; Size of this descriptor in bytes
	retlw	USB_DESCRIPTOR_CONFIGURATION	; CONFIGURATION descriptor type
Config1Len
	retlw	low ((EndConfig1-Config1)/2)	; Total length of data for this cfg
	retlw	high ((EndConfig1-Config1)/2)
	retlw	NUM_INTERFACES	; Number of interfaces in this cfg
	retlw	0x01		; Index value of this configuration
	retlw	0x00		; Configuration string index
	retlw	_DEFAULT|_SELF	; Attributes
	retlw	50		; Max power consumption (2X mA)
Interface1
	retlw	(CDCDescriptor1-Interface1)/2	; Size of this descriptor in bytes
	retlw	USB_DESCRIPTOR_INTERFACE	; INTERFACE descriptor type
	retlw	0x00		; Interface Number
	retlw	0x00		; Alternate Setting Number
	retlw	0x01		; Number of endpoints in this intf
	retlw	COMM_INTF	; Class code
	retlw	ABSTRACT_CONTROL_MODEL	; Subclass code
	retlw	V25TER		; Protocol code
	retlw	0x00		; Interface string index
CDCDescriptor1
	retlw	(CDCDescriptor2-CDCDescriptor1)/2	; Size of this descriptor in bytes
	retlw	CS_INTERFACE	; Descriptor type
	retlw	DSC_FN_HEADER	; Descriptor sub type
	retlw	0x10		; (low byte)
	retlw	0x01		; (high byte)
CDCDescriptor2
	retlw	(CDCDescriptor3-CDCDescriptor2)/2	; Size of this descriptor in bytes
	retlw	CS_INTERFACE	; Descriptor type
	retlw	DSC_FN_ACM	; Descriptor sub type
	retlw	0x02		;USB_CDC_ACM_FN_DSC_VAL	; Capabilities
CDCDescriptor3
	retlw	(CDCDescriptor4-CDCDescriptor3)/2	; Size of this descriptor in bytes
	retlw	CS_INTERFACE	; Descriptor type
	retlw	DSC_FN_UNION	; Descriptor sub type
	retlw	CDC_COMM_INTF_ID	; Master interface
	retlw	CDC_DATA_INTF_ID	; Slave interface
CDCDescriptor4
	retlw	(Endpoint2-CDCDescriptor4)/2	; Size of this descriptor in bytes
	retlw	CS_INTERFACE	; Descriptor type
	retlw	DSC_FN_CALL_MGT	; Descriptor sub type
	retlw	0x00		; Capabilities
	retlw	CDC_DATA_INTF_ID	; Data interface
Endpoint2
	retlw	(Interface2-Endpoint2)/2	; Size of this descriptor in bytes
	retlw	USB_DESCRIPTOR_ENDPOINT	; ENDPOINT descriptor type
	retlw	CDC_COMM_EP|_EP_IN	;_EP02_IN	; Endpoint Address
	retlw	_INT		; Attributes
	retlw	low (CDC_COMM_IN_EP_SIZE)	; Size of the endpoint buffer in bytes (low byte)
	retlw	high (CDC_COMM_IN_EP_SIZE)	; (high byte)
	retlw	0x02		; Polling interval (2ms)
Interface2
	retlw	(Endpoint3Out-Interface2)/2	; Size of this descriptor in bytes
	retlw	USB_DESCRIPTOR_INTERFACE	; INTERFACE descriptor type
	retlw	0x01		; Interface Number
	retlw	0x00		; Alternate Setting Number
	retlw	0x02		; Number of endpoints in this intf
	retlw	DATA_INTF	; Class code
	retlw	0x00		; Subclass code
	retlw	NO_PROTOCOL		; Protocol code
	retlw	0x00		; Interface string index
Endpoint3Out
	retlw	(Endpoint3In-Endpoint3Out)/2	; Size of this descriptor in bytes
	retlw	USB_DESCRIPTOR_ENDPOINT	; ENDPOINT descriptor type
	retlw	CDC_DATA_EP|_EP_OUT	;_EP03_OUT	; Endpoint Address
	retlw	_BULK		; Attributes
	retlw	low (CDC_DATA_OUT_EP_SIZE)	; Size of the endpoint buffer in bytes (low byte)
	retlw	high (CDC_DATA_OUT_EP_SIZE)	; (high byte)
	retlw	0x00		; Polling interval
Endpoint3In
	retlw	(EndConfig1-Endpoint3In)/2	; Size of this descriptor in bytes
	retlw	USB_DESCRIPTOR_ENDPOINT	; ENDPOINT descriptor type
	retlw	CDC_DATA_EP|_EP_IN	;_EP03_IN	; Endpoint Address
	retlw	_BULK		; Attributes
	retlw	low (CDC_DATA_IN_EP_SIZE)	; Size of the endpoint buffer in bytes (low byte)
	retlw	high (CDC_DATA_IN_EP_SIZE)	; (high byte)
	retlw	0x00		; Polling interval
EndConfig1


; String pointer table
USB_SD_Ptr
Strings
	db	low String0, high String0
	db	upper String0, 0
	db	low String1, high String1
	db	upper String1, 0
	db	low String2, high String2
	db	upper String2, 0

; Language code string descriptor
String0
	retlw   (String1-String0)/2	; Size of this descriptor in bytes
	retlw   USB_DESCRIPTOR_STRING	; STRING descriptor type
	retlw   0x09		; Language ID as defined by MS - 0x0409 (low byte)
	retlw   0x04		; (high byte)

; Manufacturer string descriptor
String1
	retlw   (String2-String1)/2	; Size of this descriptor in bytes
	retlw   USB_DESCRIPTOR_STRING	; STRING descriptor type
	retlw   'M'
	retlw   0x00
	retlw   'i'
	retlw   0x00
	retlw   'c'
	retlw   0x00
	retlw   'r'
	retlw   0x00
	retlw   'o'
	retlw   0x00
	retlw   'c'
	retlw   0x00
	retlw   'h'
	retlw   0x00
	retlw   'i'
	retlw   0x00
	retlw   'p'
	retlw   0x00
	retlw   ' '
	retlw   0x00
	retlw   'T'
	retlw   0x00
	retlw   'e'
	retlw   0x00
	retlw   'c'
	retlw   0x00
	retlw   'h'
	retlw   0x00
	retlw   'n'
	retlw   0x00
	retlw   'o'
	retlw   0x00
	retlw   'l'
	retlw   0x00
	retlw   'o'
	retlw   0x00
	retlw   'g'
	retlw   0x00
	retlw   'y'
	retlw   0x00
	retlw   ' '
	retlw   0x00
	retlw   'I'
	retlw   0x00
	retlw   'n'
	retlw   0x00
	retlw   'c'
	retlw   0x00
	retlw   '.'
	retlw   0x00

; Product string descriptor
String2
	retlw   (EndStringDescriptors-String2)/2	; Size of this descriptor in bytes
	retlw   USB_DESCRIPTOR_STRING	; STRING descriptor type
	retlw   'C'
	retlw   0x00
	retlw   'D'
	retlw   0x00
	retlw   'C'
	retlw   0x00
	retlw   ' '
	retlw   0x00
	retlw   'R'
	retlw   0x00
	retlw   'S'
	retlw   0x00
	retlw   '-'
	retlw   0x00
	retlw   '2'
	retlw   0x00
	retlw   '3'
	retlw   0x00
	retlw   '2'
	retlw   0x00
	retlw   ' '
	retlw   0x00
	retlw   'E'
	retlw   0x00
	retlw   'm'
	retlw   0x00
	retlw   'u'
	retlw   0x00
	retlw   'l'
	retlw   0x00
	retlw   'a'
	retlw   0x00
	retlw   't'
	retlw   0x00
	retlw   'i'
	retlw   0x00
	retlw   'o'
	retlw   0x00
	retlw   'n'
	retlw   0x00
	retlw   ' '
	retlw   0x00
	retlw   'D'
	retlw   0x00
	retlw   'e'
	retlw   0x00
	retlw   'm'
	retlw   0x00
	retlw   'o'
	retlw   0x00
EndStringDescriptors

	Endasm

cdc_desc_end			' End of skipped over USB assembler code
